import path from "path";
import fs from "fs";
import { app, BrowserWindow, Menu, screen, dialog, ipcMain } from "electron";
import installExtension, { VUEJS3_DEVTOOLS } from "electron-devtools-installer";

import { exec } from "child_process";
import process from "process";
import IniParser from "ini-parser";
import logger from "./logger";

const isDev = process.env.NODE_ENV === "development";
let win: BrowserWindow | null = null;
let loadWin: any = null;
const additionalData = { lockKey: "scanAdmin" };
const gotTheLock = app.requestSingleInstanceLock(additionalData);
logger.info(`gotTheLock:${gotTheLock}`);
if (!gotTheLock) {
  logger.info(`app quit => gotTheLock:${gotTheLock}`);
  app.quit();
} else {
  app.on(
    "second-instance",
    (event, commandLine, workingDirectory, additionalData) => {
      if (win) {
        if (win.isMinimized()) win.restore();
        //监听第二次双击进入应用程序的操作，如果本窗口是hide状态，并不会自动显示出来，而且会在任务管理器的后台进程里多好几个进程名
        //于是需要在监听回调里，做激活显示本窗口的操作
        win.show();
      }
    }
  );
}

function createWin() {
  const { width, height } = screen.getPrimaryDisplay().bounds;
  // Menu.setApplicationMenu(null);
  // 创建浏览器窗口
  win = new BrowserWindow({
    width: 860,
    // width: width,
    height: 520,
    // height: height,
    frame: false,
    resizable: false,
    transparent: true,
    webPreferences: {
      preload: path.resolve(__dirname, "preload.js"),
      contextIsolation: true,
      webSecurity: false,
      sandbox: false,
    },
  });

  if (isDev) {
    win.loadURL(`http://localhost:8090`);
  } else {
    win.loadFile(path.resolve(__dirname, "index.html"));
  }

  win.webContents.once("dom-ready", () => {
    win?.show();
  });
  win.webContents.on("before-input-event", (event: any, input: any) => {
    if (input.key === "F12") {
      win?.webContents.openDevTools();
    }
  });
  // loadWin.destroy();

  win.on("closed", () => {
    win = null;
  });
}

function createLoadWin() {
  Menu.setApplicationMenu(null);
  loadWin = new BrowserWindow({
    width: 840,
    height: 500,
    backgroundColor: "#222",
    frame: false,
    transparent: true,
    skipTaskbar: true,
    resizable: false,
    // webPreferences: { experimentalFeatures: true },
  });

  loadWin.loadFile(path.resolve(__dirname, "static/load/index.html"));

  loadWin.show();

  setTimeout(async () => {
    if (isDev) {
      try {
        await installExtension(VUEJS3_DEVTOOLS);
      } catch (e: any) {
        console.error("Vue Devtools failed to install:", e.toString());
      }
    }
    createWin();
  }, 2200);

  loadWin.on("closed", () => {
    loadWin = null;
  });
}

// app.isReady()
//   ? createLoadWin()
//   : app.on("ready", () => {
//       createLoadWin();
//     });

const isRunning = (query: string, cb: Function) => {
  exec(
    "cmd /c chcp 65001>nul && tasklist /FO CSV",
    (err: any, stdout: any, stderr: any) => {
      cb(stdout.toLowerCase().indexOf(query.toLowerCase()) > -1);
    }
  );
};

function watchClientIsRunning(times = 10) {
  setTimeout(() => {
    isRunning("client.exe", (status: boolean) => {
      logger.info(`watchClientIsRunning times:${times},status:${status}`);

      if (status) {
        if (times > 0) {
          watchClientIsRunning(times--);
        } else {
          logger.error(
            `意外的事情发生了：扫描端退出了，client.exe进程却还在，
            于是为了安全起见（场景1：扫描端其实没有退出，这时候唤醒登录框其实是错误的；
            场景2：扫描端退出了，client.exe进程却良久存在，
            这时候如果不唤醒登录框或者退出主程序，下次执行主程序exe就会锁住打不开），为了兼顾两种可能性，此处选择退出主程序...`
          );
          app.quit();
        }
      } else {
        win?.show();
      }
    });
  }, 500);
}

function startExe(exePath: string) {
  logger.info("主进程接收到的exe路径：", exePath);
  let checkPath = exePath.includes(".exe ")
    ? exePath.split(".exe ")[0] + ".exe"
    : exePath;
  const fileExists = fs.existsSync(checkPath);
  if (fileExists) {
    // exec(exePath, (error, stdout, stderr) => {
    //   console.log("子进程关闭了！");
    //   win?.show();
    // });
    exec(exePath, (error, stdout, stderr) => {
      if (error) {
        logger.error(`client error:${error.toString()}`);
        return;
      }
      logger.info(`client closed stdout:${stdout.toString()}`);
      logger.info(`client closed stderr:${stderr.toString()}`);
      console.log("子进程关闭了！");
      watchClientIsRunning();
    });
  } else {
    dialog.showErrorBox("tip", `${checkPath}不存在！`);
  }
}

function registEvent() {
  ipcMain.on("change-win-size", (event, args: string) => {
    // const { width, height } = screen.getPrimaryDisplay().workAreaSize;
    if (!win) return;
    const windowBounds = win.getBounds();
    const { width, height } =
      screen.getDisplayMatching(windowBounds).workAreaSize;

    let w = args === "big" ? width : 840;
    let h = args === "big" ? height : 500;
    if (args === "small") {
      win.setMinimumSize(w, h);
    }
    win.setSize(w, h);
    win.center();
  });
  ipcMain.on("window-min", () => {
    win?.minimize();
  });

  ipcMain.on("startExe", (event, args: string) => {
    startExe(args);
  });

  ipcMain.on("hide-app", () => {
    win?.hide();
  });

  ipcMain.on("get-page-size-ini", (event: any) => {
    const appPath = isDev ? process.cwd() : path.dirname(app.getPath("exe"));
    const iniFilePath = path.resolve(appPath, "page_size.ini");
    fs.readFile(iniFilePath, "utf8", (err: any, data: any) => {
      if (err) dialog.showErrorBox("tip", `page_size.ini文件不存在！`);
      const content = IniParser.parse(data);
      event.sender.send("got-page-size-ini", content);
    });
  });
}

app.on("ready", async () => {
  if (isDev) {
    try {
      await installExtension(VUEJS3_DEVTOOLS);
    } catch (e: any) {
      console.error("Vue Devtools failed to install:", e.toString());
    }
  }

  logger.info(`app ready`);
  registEvent();

  createWin();
});
